/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
#ifndef ENTITY_H_
#define ENTITY_H_

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "Object.h"
#include "Sprite.h"
#include "StateMachine.h"
#include "Telegram.h"
#include "MiscStructs.h"



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#define Entity_METHODS								\
	Object_METHODS;									\
	__VIRTUAL_METHOD(update);						\
	__VIRTUAL_METHOD(render);						\
	__VIRTUAL_METHOD(handleMessage);				\
	__VIRTUAL_METHOD(write);						\
	__VIRTUAL_METHOD(setPosition);					\
	__VIRTUAL_METHOD(getScale);						\
	__VIRTUAL_METHOD(getInGameState);				\
	__VIRTUAL_METHOD(updateLayer);					\
	__VIRTUAL_METHOD(setLayer);						\
	__VIRTUAL_METHOD(isVisible);					\
	__VIRTUAL_METHOD(setExtraInfo);

#define Entity_SET_VTABLE(ClassName)					\
	Object_SET_VTABLE(ClassName);						\
	__VIRTUAL_SET(ClassName, Entity, update);			\
	__VIRTUAL_SET(ClassName, Entity, render);			\
	__VIRTUAL_SET(ClassName, Entity, handleMessage);	\
	__VIRTUAL_SET(ClassName, Entity, write);			\
	__VIRTUAL_SET(ClassName, Entity, setPosition);		\
	__VIRTUAL_SET(ClassName, Entity, getScale);			\
	__VIRTUAL_SET(ClassName, Entity, getInGameState);	\
	__VIRTUAL_SET(ClassName, Entity, updateLayer);		\
	__VIRTUAL_SET(ClassName, Entity, setLayer);			\
	__VIRTUAL_SET(ClassName, Entity, isVisible);		\
	__VIRTUAL_SET(ClassName, Entity, setExtraInfo);
	

// A Entity which represent a generic object inside a Stage
__CLASS(Entity);

#define Entity_ATTRIBUTES							\
													\
	/* it is derivated from*/						\
	Object_ATTRIBUTES								\
													\
	/* entity's id */								\
	int ID;											\
													\
	/* a pointer to a Sprite */						\
	Sprite sprite;									\
													\
	/* 3d position */								\
	VBVec3D position;




/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S ROM DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// defines a Entity in ROM memory
typedef const struct EntityDefinition{

	// the class type
	void* allocator;
	
	// the sprite
	SpriteDefinition spriteDefinition;
	
}EntityDefinition;

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
void Entity_constructor(Entity this, EntityDefinition* entityDefinition, int ID);

// class's destructor
void Entity_destructor(Entity this);

// process extra info in intialization
void Entity_setExtraInfo(Entity this, void* extraInfo);

// update
void Entity_update(Entity this);

//set class's animation
void Entity_setAnimation(Entity this, void (*animation)(Entity this));

//get map's type
int Entity_getMapType(Entity this);

//retrieve class's map's mode
int Entity_getMapMode(Entity this);

//grite Entity to graphic memory
void Entity_write(Entity this);

// allocate a write in graphic memory again
void Entity_resetMemoryState(Entity this, int worldLayer);

//render class
void Entity_render(Entity this);

//set class's position
void Entity_setPosition(Entity this, VBVec3D position);

//retrieve class's in game index
int Entity_getID(Entity this);


//retrieve class's scale
Scale Entity_getScale(Entity this);

//retrieve class's map's paralax
int Entity_getMapParallax(Entity this);

// retrieve Entity's friction
float Entity_getFrictionFactor(Entity this);

// set graphical gap
void Entity_setCollisionGap(Entity this, int upGap, int downGap, int leftGap, int rightGap);

// retrieve position
VBVec3D Entity_getPosition(Entity this);

// retrieve in game type
int Entity_getInGameType(Entity this);

// retrieve sprite
Sprite Entity_getSprite(Entity this);

// process a telegram
int Entity_handleMessage(Entity this, Telegram telegram);

// get width
int Entity_getWidth(Entity this);

// get height
int Entity_getHeight(Entity this);

// whether it is visible
int Entity_isVisible(Entity this, int pad);

// determine if the entity is outside the game
int Entity_isOutsideGame(Entity this);

// retrieve state when unloading the entity 
int Entity_getInGameState(Entity this);

// update sprite's world layer 
void Entity_updateLayer(Entity this, int worldLayerIncrement);

// set sprite's world layer 
void Entity_setLayer(Entity this, int worldLayer);

#endif /*ENTITY_H_*/
