/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
#ifndef RECT_H_
#define RECT_H_

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "Object.h"
#include "Shape.h"



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#define Rect_METHODS								\
	Shape_METHODS;

#define Rect_SET_VTABLE(ClassName)					\
	Shape_SET_VTABLE(ClassName);					\
	__VIRTUAL_SET(ClassName, Rect, overlaps);		\
	__VIRTUAL_SET(ClassName, Rect, setup);			\
	__VIRTUAL_SET(ClassName, Rect, prepare);

// A Rect which represent a generic object inside a Stage
__CLASS(Rect);

typedef struct Rectangle{

	/* left upper corner */
	int x0;
	int y0;
			
	/* right down corner */
	int x1;					
	int y1;

}Rectangle;

#define Rect_ATTRIBUTES								\
													\
	/* it is derivated from*/						\
	Shape_ATTRIBUTES								\
													\
	/* the rectangle */								\
	Rectangle rectangle;							\
													\
	/* the rectangle to check */					\
	Rectangle positionedRectangle;
	




/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// class's allocator
Rect Rect_new(InGameEntity owner, int deep);


// class's destructor
void Rect_destructor(Rect this);

// check if overlaps with other shape
int Rect_overlaps(Rect this, Shape shape, int axisMovement);

// setup the rectangle
void Rect_setup(Rect this, const VBVec3D* const position);

// prepare the shape to be checked
void Rect_prepare(Rect this);

#endif /*RECT_H_*/
