/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef STATEMACHINE_H_
#define STATEMACHINE_H_


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include <libgccvb.h>
#include "Telegram.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// just a macro to make it easier to contruct a state, and to allow future changes
#define __STATE_CONSTRUCT(ClassName, SuperClass)						\
																		\
	/* call super constructor */										\
	SuperClass ## _constructor((SuperClass)this, 						\
								(void*)ClassName ## _enter,				\
								(void*)ClassName ## _execute,			\
								(void*)ClassName ## _exit,				\
								(void*)ClassName ## _pause,				\
								(void*)ClassName ## _resume,			\
								(void*)ClassName ## _handleMessage);



// declare the virtual methods
#define State_METHODS								\
	Object_METHODS;									

// declare the virtual methods which are redefined
#define State_SET_VTABLE(ClassName)						\
	Object_SET_VTABLE(ClassName)							

// declare a Sprite, which holds a texture and a drawing specification
__CLASS(State);

// a generic state
#define State_ATTRIBUTES												\
																		\
	/* it is derivated from */											\
	Object_ATTRIBUTES													\
																		\
	/* */ 																\
	void (*enter)(State this, void* owner);								\
																		\
	/* */ 																\
	void (*execute)(State this, void* owner);							\
																		\
	/* */ 																\
	void (*exit)(State this, void* owner);								\
																		\
	/* */ 																\
	void (*pause)(State this, void* owner);								\
																		\
	/* */ 																\
	void (*resume)(State this, void* owner);							\
																		\
	/* */ 																\
	int (*onMessage)(State this, void* owner, Telegram telegram);		\
																		\
	/* */ 																\
	/*int (*getType)(State this);*/
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//class's constructor
void State_constructor(State this, void (*enter)(State this, void* owner) , 
						void (*execute)(State this, void* owner) , 
						void (*exit)(State this, void* owner), 	
						void (*pause)(State this, void* owner),
						void (*resume)(State this, void* owner),
						int (*onMessage)(State this, void* owner, 
						Telegram telegram));

//class's destructor
void State_destructor(State this);



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */




// declare the virtual methods
#define StateMachine_METHODS								\
	Object_METHODS;									

// declare the virtual methods which are redefined
#define StateMachine_SET_VTABLE(ClassName)						\
	Object_SET_VTABLE(ClassName)							


__CLASS(StateMachine);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// class's allocator
StateMachine StateMachine_new(void* owner);

// allocate memory and call the constructor
void StateMachine_constructor(StateMachine this, void* owner);

// class's destructor
void StateMachine_destructor(StateMachine this);

// update state
void StateMachine_update(StateMachine this);

// change state
void StateMachine_swapState(StateMachine this, State newState);

// push a state in the stack
void StateMachine_pushState(StateMachine this, State newState);

// pop a state fromt the stack
void StateMachine_popState(StateMachine this);

// return to previous state
void StateMachine_returnToPreviousState(StateMachine this);

// change to a global state 
void StateMachine_changeToGlobal(StateMachine this, State globalState);

// return to previous state
int StateMachine_handleMessage(StateMachine this, Telegram telegram);

// returns true if the current state's type is equal to the type of the
// class passed as a parameter. 
int StateMachine_isInState(StateMachine this, State state);

// set owner
void StateMachine_setOwner(StateMachine this, void* owner);

#endif /*SCROLL_H_*/
